@@ -39,7 +39,7 @@ use std::str::FromStr;
 use anyhow::Result;
 use url::Url;
 
-use distribution_filename::{DistFilename, SourceDistFilename, WheelFilename};
+use distribution_filename::WheelFilename;
 use pep440_rs::Version;
 use pep508_rs::{Pep508Url, VerbatimUrl};
 use uv_git::GitUrl;
@@ -158,12 +158,42 @@ pub enum SourceDist {
 
 /// A built distribution (wheel) that exists in a registry, like `PyPI`.
 #[derive(Debug, Clone)]
-pub struct RegistryBuiltDist {
+pub struct RegistryBuiltWheel {
     pub filename: WheelFilename,
     pub file: Box<File>,
     pub index: IndexUrl,
 }
 
+/// A built distribution (wheel) that exists in a registry, like `PyPI`.
+#[derive(Debug, Clone)]
+pub struct RegistryBuiltDist {
+    /// All wheels associated with this distribution. It is guaranteed
+    /// that there is at least one wheel.
+    pub wheels: Vec<RegistryBuiltWheel>,
+    /// The "best" wheel selected based on the current wheel tag
+    /// environment.
+    ///
+    /// This is guaranteed to point into a valid entry in `wheels`.
+    pub best_wheel_index: usize,
+    /// A source distribution if one exists for this distribution.
+    ///
+    /// It is possible for this to be `None`. For example, when a distribution
+    /// has no source distribution, or if it does have one but isn't compatible
+    /// with the user configuration. (e.g., If `Requires-Python` isn't
+    /// compatible with the installed/target Python versions, or if something
+    /// like `--exclude-newer` was used.)
+    pub sdist: Option<RegistrySourceDist>,
+    // Ideally, this type would have an index URL on it, and the
+    // `RegistryBuiltDist` and `RegistrySourceDist` types would *not* have an
+    // index URL on them. Alas, the --find-links feature makes it technically
+    // possible for the indexes to diverge across wheels/sdists in the same
+    // distribution.
+    //
+    // Note though that at time of writing, when generating a universal lock
+    // file, we require that all index URLs across wheels/sdists for a single
+    // distribution are equivalent.
+}
+
 /// A built distribution (wheel) that exists at an arbitrary URL.
 #[derive(Debug, Clone)]
 pub struct DirectUrlBuiltDist {
@@ -172,8 +202,7 @@ pub struct DirectUrlBuiltDist {
     pub filename: WheelFilename,
     /// The URL without the subdirectory fragment.
     pub location: Url,
-    pub subdirectory: Option<PathBuf>,
-    /// The URL with the subdirectory fragment.
+    /// The URL as it was provided by the user.
     pub url: VerbatimUrl,
 }
 
@@ -181,16 +210,27 @@ pub struct DirectUrlBuiltDist {
 #[derive(Debug, Clone)]
 pub struct PathBuiltDist {
     pub filename: WheelFilename,
-    pub url: VerbatimUrl,
+    /// The path to the wheel.
     pub path: PathBuf,
+    /// The URL as it was provided by the user.
+    pub url: VerbatimUrl,
 }
 
 /// A source distribution that exists in a registry, like `PyPI`.
 #[derive(Debug, Clone)]
 pub struct RegistrySourceDist {
-    pub filename: SourceDistFilename,
+    pub name: PackageName,
+    pub version: Version,
     pub file: Box<File>,
     pub index: IndexUrl,
+    /// When an sdist is selected, it may be the case that there were
+    /// available wheels too. There are many reasons why a wheel might not
+    /// have been chosen (maybe none available are compatible with the
+    /// current environment), but we still want to track that they exist. In
+    /// particular, for generating a universal lock file, we do not want to
+    /// skip emitting wheels to the lock file just because the host generating
+    /// the lock file didn't have any compatible wheels available.
+    pub wheels: Vec<RegistryBuiltWheel>,
 }
 
 /// A source distribution that exists at an arbitrary URL.
@@ -201,8 +241,9 @@ pub struct DirectUrlSourceDist {
     pub name: PackageName,
     /// The URL without the subdirectory fragment.
     pub location: Url,
+    /// The subdirectory within the archive in which the source distribution is located.
     pub subdirectory: Option<PathBuf>,
-    /// The URL with the subdirectory fragment.
+    /// The URL as it was provided by the user, including the subdirectory fragment.
     pub url: VerbatimUrl,
 }
 
@@ -212,8 +253,9 @@ pub struct GitSourceDist {
     pub name: PackageName,
     /// The URL without the revision and subdirectory fragment.
     pub git: Box<GitUrl>,
+    /// The subdirectory within the Git repository in which the source distribution is located.
     pub subdirectory: Option<PathBuf>,
-    /// The URL with the revision and subdirectory fragment.
+    /// The URL as it was provided by the user, including the revision and subdirectory fragment.
     pub url: VerbatimUrl,
 }
 
@@ -221,40 +263,25 @@ pub struct GitSourceDist {
 #[derive(Debug, Clone)]
 pub struct PathSourceDist {
     pub name: PackageName,
-    pub url: VerbatimUrl,
+    /// The path to the archive.
     pub path: PathBuf,
+    /// The URL as it was provided by the user.
+    pub url: VerbatimUrl,
 }
 
 /// A source distribution that exists in a local directory.
 #[derive(Debug, Clone)]
 pub struct DirectorySourceDist {
     pub name: PackageName,
-    pub url: VerbatimUrl,
+    /// The path to the directory.
     pub path: PathBuf,
+    /// Whether the package should be installed in editable mode.
     pub editable: bool,
+    /// The URL as it was provided by the user.
+    pub url: VerbatimUrl,
 }
 
 impl Dist {
-    /// Create a [`Dist`] for a registry-based distribution.
-    pub fn from_registry(filename: DistFilename, file: File, index: IndexUrl) -> Self {
-        match filename {
-            DistFilename::WheelFilename(filename) => {
-                Self::Built(BuiltDist::Registry(RegistryBuiltDist {
-                    filename,
-                    file: Box::new(file),
-                    index,
-                }))
-            }
-            DistFilename::SourceDistFilename(filename) => {
-                Self::Source(SourceDist::Registry(RegistrySourceDist {
-                    filename,
-                    file: Box::new(file),
-                    index,
-                }))
-            }
-        }
-    }
-
     /// A remote built distribution (`.whl`) or source distribution from a `http://` or `https://`
     /// URL.
     pub fn from_http_url(
@@ -280,7 +307,6 @@ impl Dist {
             Ok(Self::Built(BuiltDist::DirectUrl(DirectUrlBuiltDist {
                 filename,
                 location,
-                subdirectory,
                 url,
             })))
         } else {
@@ -313,9 +339,9 @@ impl Dist {
         if path.is_dir() {
             Ok(Self::Source(SourceDist::Directory(DirectorySourceDist {
                 name,
-                url,
                 path,
                 editable,
+                url,
             })))
         } else if path
             .extension()
@@ -337,8 +363,8 @@ impl Dist {
 
             Ok(Self::Built(BuiltDist::Path(PathBuiltDist {
                 filename,
-                url,
                 path,
+                url,
             })))
         } else {
             if editable {
@@ -347,8 +373,8 @@ impl Dist {
 
             Ok(Self::Source(SourceDist::Path(PathSourceDist {
                 name,
-                url,
                 path,
+                url,
             })))
         }
     }
@@ -428,7 +454,7 @@ impl BuiltDist {
     /// Returns the [`IndexUrl`], if the distribution is from a registry.
     pub fn index(&self) -> Option<&IndexUrl> {
         match self {
-            Self::Registry(registry) => Some(&registry.index),
+            Self::Registry(registry) => Some(&registry.best_wheel().index),
             Self::DirectUrl(_) => None,
             Self::Path(_) => None,
         }
@@ -437,14 +463,14 @@ impl BuiltDist {
     /// Returns the [`File`] instance, if this distribution is from a registry.
     pub fn file(&self) -> Option<&File> {
         match self {
-            Self::Registry(registry) => Some(&registry.file),
+            Self::Registry(registry) => Some(&registry.best_wheel().file),
             Self::DirectUrl(_) | Self::Path(_) => None,
         }
     }
 
     pub fn version(&self) -> &Version {
         match self {
-            Self::Registry(wheel) => &wheel.filename.version,
+            Self::Registry(wheels) => &wheels.best_wheel().filename.version,
             Self::DirectUrl(wheel) => &wheel.filename.version,
             Self::Path(wheel) => &wheel.filename.version,
         }
@@ -470,7 +496,7 @@ impl SourceDist {
 
     pub fn version(&self) -> Option<&Version> {
         match self {
-            Self::Registry(source_dist) => Some(&source_dist.filename.version),
+            Self::Registry(source_dist) => Some(&source_dist.version),
             Self::DirectUrl(_) | Self::Git(_) | Self::Path(_) | Self::Directory(_) => None,
         }
     }
@@ -493,12 +519,25 @@ impl SourceDist {
     }
 }
 
-impl Name for RegistryBuiltDist {
+impl RegistryBuiltDist {
+    /// Returns the best or "most compatible" wheel in this distribution.
+    pub fn best_wheel(&self) -> &RegistryBuiltWheel {
+        &self.wheels[self.best_wheel_index]
+    }
+}
+
+impl Name for RegistryBuiltWheel {
     fn name(&self) -> &PackageName {
         &self.filename.name
     }
 }
 
+impl Name for RegistryBuiltDist {
+    fn name(&self) -> &PackageName {
+        self.best_wheel().name()
+    }
+}
+
 impl Name for DirectUrlBuiltDist {
     fn name(&self) -> &PackageName {
         &self.filename.name
@@ -513,7 +552,7 @@ impl Name for PathBuiltDist {
 
 impl Name for RegistrySourceDist {
     fn name(&self) -> &PackageName {
-        &self.filename.name
+        &self.name
     }
 }
 
@@ -572,12 +611,18 @@ impl Name for Dist {
     }
 }
 
-impl DistributionMetadata for RegistryBuiltDist {
+impl DistributionMetadata for RegistryBuiltWheel {
     fn version_or_url(&self) -> VersionOrUrlRef {
         VersionOrUrlRef::Version(&self.filename.version)
     }
 }
 
+impl DistributionMetadata for RegistryBuiltDist {
+    fn version_or_url(&self) -> VersionOrUrlRef {
+        self.best_wheel().version_or_url()
+    }
+}
+
 impl DistributionMetadata for DirectUrlBuiltDist {
     fn version_or_url(&self) -> VersionOrUrlRef {
         VersionOrUrlRef::Url(&self.url)
@@ -592,7 +637,7 @@ impl DistributionMetadata for PathBuiltDist {
 
 impl DistributionMetadata for RegistrySourceDist {
     fn version_or_url(&self) -> VersionOrUrlRef {
-        VersionOrUrlRef::Version(&self.filename.version)
+        VersionOrUrlRef::Version(&self.version)
     }
 }
 
@@ -664,13 +709,15 @@ impl RemoteSource for File {
 impl RemoteSource for Url {
     fn filename(&self) -> Result<Cow<'_, str>, Error> {
         // Identify the last segment of the URL as the filename.
-        let filename = self
+        let path_segments = self
             .path_segments()
-            .and_then(Iterator::last)
-            .ok_or_else(|| Error::UrlFilename(self.clone()))?;
+            .ok_or_else(|| Error::MissingPathSegments(self.clone()))?;
+
+        // This is guaranteed by the contract of `Url::path_segments`.
+        let last = path_segments.last().expect("path segments is non-empty");
 
         // Decode the filename, which may be percent-encoded.
-        let filename = urlencoding::decode(filename)?;
+        let filename = urlencoding::decode(last)?;
 
         Ok(filename)
     }
@@ -680,7 +727,7 @@ impl RemoteSource for Url {
     }
 }
 
-impl RemoteSource for RegistryBuiltDist {
+impl RemoteSource for RegistryBuiltWheel {
     fn filename(&self) -> Result<Cow<'_, str>, Error> {
         self.file.filename()
     }
@@ -690,6 +737,16 @@ impl RemoteSource for RegistryBuiltDist {
     }
 }
 
+impl RemoteSource for RegistryBuiltDist {
+    fn filename(&self) -> Result<Cow<'_, str>, Error> {
+        self.best_wheel().filename()
+    }
+
+    fn size(&self) -> Option<u64> {
+        self.best_wheel().size()
+    }
+}
+
 impl RemoteSource for RegistrySourceDist {
     fn filename(&self) -> Result<Cow<'_, str>, Error> {
         self.file.filename()
@@ -892,7 +949,7 @@ impl Identifier for FileLocation {
     }
 }
 
-impl Identifier for RegistryBuiltDist {
+impl Identifier for RegistryBuiltWheel {
     fn distribution_id(&self) -> DistributionId {
         self.file.distribution_id()
     }
@@ -902,6 +959,16 @@ impl Identifier for RegistryBuiltDist {
     }
 }
 
+impl Identifier for RegistryBuiltDist {
+    fn distribution_id(&self) -> DistributionId {
+        self.best_wheel().distribution_id()
+    }
+
+    fn resource_id(&self) -> ResourceId {
+        self.best_wheel().resource_id()
+    }
+}
+
 impl Identifier for RegistrySourceDist {
     fn distribution_id(&self) -> DistributionId {
         self.file.distribution_id()
