@@ -4,7 +4,9 @@ use pep440_rs::VersionSpecifiers;
 use platform_tags::{IncompatibleTag, TagPriority};
 use pypi_types::{HashDigest, Yanked};
 
-use crate::{Dist, InstalledDist, ResolvedDistRef};
+use crate::{
+    InstalledDist, RegistryBuiltDist, RegistryBuiltWheel, RegistrySourceDist, ResolvedDistRef,
+};
 
 /// A collection of distributions that have been filtered by relevance.
 #[derive(Debug, Default, Clone)]
@@ -14,9 +16,12 @@ pub struct PrioritizedDist(Box<PrioritizedDistInner>);
 #[derive(Debug, Default, Clone)]
 struct PrioritizedDistInner {
     /// The highest-priority source distribution. Between compatible source distributions this priority is arbitrary.
-    source: Option<(Dist, SourceDistCompatibility)>,
-    /// The highest-priority wheel.
-    wheel: Option<(Dist, WheelCompatibility)>,
+    source: Option<(RegistrySourceDist, SourceDistCompatibility)>,
+    /// The highest-priority wheel index. When present, it is
+    /// guaranteed to be a valid index into `wheels`.
+    best_wheel_index: Option<usize>,
+    /// The set of all wheels associated with this distribution.
+    wheels: Vec<(RegistryBuiltWheel, WheelCompatibility)>,
     /// The hashes for each distribution.
     hashes: Vec<HashDigest>,
 }
@@ -27,14 +32,30 @@ pub enum CompatibleDist<'a> {
     /// The distribution is already installed and can be used.
     InstalledDist(&'a InstalledDist),
     /// The distribution should be resolved and installed using a source distribution.
-    SourceDist(&'a Dist),
+    SourceDist {
+        /// The source distribution that should be used.
+        sdist: &'a RegistrySourceDist,
+        /// The prioritized distribution that the sdist came from.
+        prioritized: &'a PrioritizedDist,
+    },
     /// The distribution should be resolved and installed using a wheel distribution.
-    CompatibleWheel(&'a Dist, TagPriority),
+    CompatibleWheel {
+        /// The wheel that should be used.
+        wheel: &'a RegistryBuiltWheel,
+        /// The platform priority associated with the wheel.
+        priority: TagPriority,
+        /// The prioritized distribution that the wheel came from.
+        prioritized: &'a PrioritizedDist,
+    },
     /// The distribution should be resolved using an incompatible wheel distribution, but
     /// installed using a source distribution.
     IncompatibleWheel {
-        source_dist: &'a Dist,
-        wheel: &'a Dist,
+        /// The sdist to be used during installation.
+        sdist: &'a RegistrySourceDist,
+        /// The wheel to be used during resolution.
+        wheel: &'a RegistryBuiltWheel,
+        /// The prioritized distribution that the wheel and sdist came from.
+        prioritized: &'a PrioritizedDist,
     },
 }
 
@@ -53,59 +74,59 @@ impl Display for IncompatibleDist {
         match self {
             Self::Wheel(incompatibility) => match incompatibility {
                 IncompatibleWheel::NoBinary => {
-                    f.write_str("no source distribution is available and using wheels is disabled")
+                    f.write_str("has no available source distribution and using wheels is disabled")
                 }
                 IncompatibleWheel::Tag(tag) => match tag {
                     IncompatibleTag::Invalid => {
-                        f.write_str("no wheels are available with valid tags")
-                    }
-                    IncompatibleTag::Python => {
-                        f.write_str("no wheels are available with a matching Python implementation")
+                        f.write_str("has no wheels are available with valid tags")
                     }
+                    IncompatibleTag::Python => f.write_str(
+                        "has no wheels are available with a matching Python implementation",
+                    ),
                     IncompatibleTag::Abi => {
-                        f.write_str("no wheels are available with a matching Python ABI")
+                        f.write_str("has no wheels are available with a matching Python ABI")
                     }
                     IncompatibleTag::Platform => {
-                        f.write_str("no wheels are available with a matching platform")
+                        f.write_str("has no wheels are available with a matching platform")
                     }
                 },
                 IncompatibleWheel::Yanked(yanked) => match yanked {
-                    Yanked::Bool(_) => f.write_str("it was yanked"),
+                    Yanked::Bool(_) => f.write_str("was yanked"),
                     Yanked::Reason(reason) => write!(
                         f,
-                        "it was yanked (reason: {})",
+                        "was yanked (reason: {})",
                         reason.trim().trim_end_matches('.')
                     ),
                 },
                 IncompatibleWheel::ExcludeNewer(ts) => match ts {
-                    Some(_) => f.write_str("it was published after the exclude newer time"),
-                    None => f.write_str("it has no publish time"),
+                    Some(_) => f.write_str("was published after the exclude newer time"),
+                    None => f.write_str("has no publish time"),
                 },
                 IncompatibleWheel::RequiresPython(python) => {
-                    write!(f, "it requires at python {python}")
+                    write!(f, "requires at python {python}")
                 }
             },
             Self::Source(incompatibility) => match incompatibility {
                 IncompatibleSource::NoBuild => {
-                    f.write_str("no wheels are usable and building from source is disabled")
+                    f.write_str("has no usable wheels and building from source is disabled")
                 }
                 IncompatibleSource::Yanked(yanked) => match yanked {
-                    Yanked::Bool(_) => f.write_str("it was yanked"),
+                    Yanked::Bool(_) => f.write_str("was yanked"),
                     Yanked::Reason(reason) => write!(
                         f,
-                        "it was yanked (reason: {})",
+                        "was yanked (reason: {})",
                         reason.trim().trim_end_matches('.')
                     ),
                 },
                 IncompatibleSource::ExcludeNewer(ts) => match ts {
-                    Some(_) => f.write_str("it was published after the exclude newer time"),
-                    None => f.write_str("it has no publish time"),
+                    Some(_) => f.write_str("was published after the exclude newer time"),
+                    None => f.write_str("has no publish time"),
                 },
                 IncompatibleSource::RequiresPython(python) => {
-                    write!(f, "it requires python {python}")
+                    write!(f, "requires python {python}")
                 }
             },
-            Self::Unavailable => f.write_str("no distributions are available"),
+            Self::Unavailable => f.write_str("has no available distributions"),
         }
     }
 }
@@ -113,7 +134,7 @@ impl Display for IncompatibleDist {
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub enum WheelCompatibility {
     Incompatible(IncompatibleWheel),
-    Compatible(Hash, TagPriority),
+    Compatible(HashComparison, TagPriority),
 }
 
 #[derive(Debug, PartialEq, Eq, Clone)]
@@ -128,7 +149,7 @@ pub enum IncompatibleWheel {
 #[derive(Debug, Clone, PartialEq, Eq)]
 pub enum SourceDistCompatibility {
     Incompatible(IncompatibleSource),
-    Compatible(Hash),
+    Compatible(HashComparison),
 }
 
 #[derive(Debug, PartialEq, Eq, Clone)]
@@ -140,7 +161,7 @@ pub enum IncompatibleSource {
 }
 
 #[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord)]
-pub enum Hash {
+pub enum HashComparison {
     /// The hash is present, but does not match the expected value.
     Mismatched,
     /// The hash is missing.
@@ -152,12 +173,13 @@ pub enum Hash {
 impl PrioritizedDist {
     /// Create a new [`PrioritizedDist`] from the given wheel distribution.
     pub fn from_built(
-        dist: Dist,
+        dist: RegistryBuiltWheel,
         hashes: Vec<HashDigest>,
         compatibility: WheelCompatibility,
     ) -> Self {
         Self(Box::new(PrioritizedDistInner {
-            wheel: Some((dist, compatibility)),
+            best_wheel_index: Some(0),
+            wheels: vec![(dist, compatibility)],
             source: None,
             hashes,
         }))
@@ -165,12 +187,13 @@ impl PrioritizedDist {
 
     /// Create a new [`PrioritizedDist`] from the given source distribution.
     pub fn from_source(
-        dist: Dist,
+        dist: RegistrySourceDist,
         hashes: Vec<HashDigest>,
         compatibility: SourceDistCompatibility,
     ) -> Self {
         Self(Box::new(PrioritizedDistInner {
-            wheel: None,
+            best_wheel_index: None,
+            wheels: vec![],
             source: Some((dist, compatibility)),
             hashes,
         }))
@@ -179,26 +202,26 @@ impl PrioritizedDist {
     /// Insert the given built distribution into the [`PrioritizedDist`].
     pub fn insert_built(
         &mut self,
-        dist: Dist,
+        dist: RegistryBuiltWheel,
         hashes: Vec<HashDigest>,
         compatibility: WheelCompatibility,
     ) {
         // Track the highest-priority wheel.
-        if let Some((.., existing_compatibility)) = &self.0.wheel {
+        if let Some((.., existing_compatibility)) = self.best_wheel() {
             if compatibility.is_more_compatible(existing_compatibility) {
-                self.0.wheel = Some((dist, compatibility));
+                self.0.best_wheel_index = Some(self.0.wheels.len());
             }
         } else {
-            self.0.wheel = Some((dist, compatibility));
+            self.0.best_wheel_index = Some(self.0.wheels.len());
         }
-
+        self.0.wheels.push((dist, compatibility));
         self.0.hashes.extend(hashes);
     }
 
     /// Insert the given source distribution into the [`PrioritizedDist`].
     pub fn insert_source(
         &mut self,
-        dist: Dist,
+        dist: RegistrySourceDist,
         hashes: Vec<HashDigest>,
         compatibility: SourceDistCompatibility,
     ) {
@@ -216,21 +239,35 @@ impl PrioritizedDist {
 
     /// Return the highest-priority distribution for the package version, if any.
     pub fn get(&self) -> Option<CompatibleDist> {
-        match (&self.0.wheel, &self.0.source) {
-            // If both are compatible, break ties based on the hash.
+        let best_wheel = self.0.best_wheel_index.map(|i| &self.0.wheels[i]);
+        match (&best_wheel, &self.0.source) {
+            // If both are compatible, break ties based on the hash outcome. For example, prefer a
+            // source distribution with a matching hash over a wheel with a mismatched hash. When
+            // the outcomes are equivalent (e.g., both have a matching hash), prefer the wheel.
             (
                 Some((wheel, WheelCompatibility::Compatible(wheel_hash, tag_priority))),
-                Some((source_dist, SourceDistCompatibility::Compatible(source_hash))),
+                Some((sdist, SourceDistCompatibility::Compatible(sdist_hash))),
             ) => {
-                if source_hash > wheel_hash {
-                    Some(CompatibleDist::SourceDist(source_dist))
+                if sdist_hash > wheel_hash {
+                    Some(CompatibleDist::SourceDist {
+                        sdist,
+                        prioritized: self,
+                    })
                 } else {
-                    Some(CompatibleDist::CompatibleWheel(wheel, *tag_priority))
+                    Some(CompatibleDist::CompatibleWheel {
+                        wheel,
+                        priority: *tag_priority,
+                        prioritized: self,
+                    })
                 }
             }
             // Prefer the highest-priority, platform-compatible wheel.
             (Some((wheel, WheelCompatibility::Compatible(_, tag_priority))), _) => {
-                Some(CompatibleDist::CompatibleWheel(wheel, *tag_priority))
+                Some(CompatibleDist::CompatibleWheel {
+                    wheel,
+                    priority: *tag_priority,
+                    prioritized: self,
+                })
             }
             // If we have a compatible source distribution and an incompatible wheel, return the
             // wheel. We assume that all distributions have the same metadata for a given package
@@ -238,37 +275,42 @@ impl PrioritizedDist {
             // using the wheel is faster.
             (
                 Some((wheel, WheelCompatibility::Incompatible(_))),
-                Some((source_dist, SourceDistCompatibility::Compatible(_))),
-            ) => Some(CompatibleDist::IncompatibleWheel { source_dist, wheel }),
+                Some((sdist, SourceDistCompatibility::Compatible(_))),
+            ) => Some(CompatibleDist::IncompatibleWheel {
+                sdist,
+                wheel,
+                prioritized: self,
+            }),
             // Otherwise, if we have a source distribution, return it.
-            (None, Some((source_dist, SourceDistCompatibility::Compatible(_)))) => {
-                Some(CompatibleDist::SourceDist(source_dist))
+            (None, Some((sdist, SourceDistCompatibility::Compatible(_)))) => {
+                Some(CompatibleDist::SourceDist {
+                    sdist,
+                    prioritized: self,
+                })
             }
             _ => None,
         }
     }
 
-    /// Return the incompatible source distribution, if any.
-    pub fn incompatible_source(&self) -> Option<(&Dist, &IncompatibleSource)> {
+    /// Return the incompatibility for the best source distribution, if any.
+    pub fn incompatible_source(&self) -> Option<&IncompatibleSource> {
         self.0
             .source
             .as_ref()
-            .and_then(|(dist, compatibility)| match compatibility {
+            .and_then(|(_, compatibility)| match compatibility {
                 SourceDistCompatibility::Compatible(_) => None,
-                SourceDistCompatibility::Incompatible(incompatibility) => {
-                    Some((dist, incompatibility))
-                }
+                SourceDistCompatibility::Incompatible(incompatibility) => Some(incompatibility),
             })
     }
 
-    /// Return the incompatible built distribution, if any.
-    pub fn incompatible_wheel(&self) -> Option<(&Dist, &IncompatibleWheel)> {
+    /// Return the incompatibility for the best wheel, if any.
+    pub fn incompatible_wheel(&self) -> Option<&IncompatibleWheel> {
         self.0
-            .wheel
-            .as_ref()
-            .and_then(|(dist, compatibility)| match compatibility {
+            .best_wheel_index
+            .map(|i| &self.0.wheels[i])
+            .and_then(|(_, compatibility)| match compatibility {
                 WheelCompatibility::Compatible(_, _) => None,
-                WheelCompatibility::Incompatible(incompatibility) => Some((dist, incompatibility)),
+                WheelCompatibility::Incompatible(incompatibility) => Some(incompatibility),
             })
     }
 
@@ -280,7 +322,48 @@ impl PrioritizedDist {
     /// Returns true if and only if this distribution does not contain any
     /// source distributions or wheels.
     pub fn is_empty(&self) -> bool {
-        self.0.source.is_none() && self.0.wheel.is_none()
+        self.0.source.is_none() && self.0.wheels.is_empty()
+    }
+
+    /// If this prioritized dist has at least one wheel, then this creates
+    /// a built distribution with the best wheel in this prioritized dist.
+    pub fn built_dist(&self) -> Option<RegistryBuiltDist> {
+        let best_wheel_index = self.0.best_wheel_index?;
+        let wheels = self
+            .0
+            .wheels
+            .iter()
+            .map(|(wheel, _)| wheel.clone())
+            .collect();
+        let sdist = self.0.source.as_ref().map(|(sdist, _)| sdist.clone());
+        Some(RegistryBuiltDist {
+            wheels,
+            best_wheel_index,
+            sdist,
+        })
+    }
+
+    /// If this prioritized dist has an sdist, then this creates a source
+    /// distribution.
+    pub fn source_dist(&self) -> Option<RegistrySourceDist> {
+        let mut sdist = self.0.source.as_ref().map(|(sdist, _)| sdist.clone())?;
+        assert!(
+            sdist.wheels.is_empty(),
+            "source distribution should not have any wheels yet"
+        );
+        sdist.wheels = self
+            .0
+            .wheels
+            .iter()
+            .map(|(wheel, _)| wheel.clone())
+            .collect();
+        Some(sdist)
+    }
+
+    /// Returns the "best" wheel in this prioritized distribution, if one
+    /// exists.
+    pub fn best_wheel(&self) -> Option<&(RegistryBuiltWheel, WheelCompatibility)> {
+        self.0.best_wheel_index.map(|i| &self.0.wheels[i])
     }
 }
 
@@ -289,12 +372,15 @@ impl<'a> CompatibleDist<'a> {
     pub fn for_resolution(&self) -> ResolvedDistRef<'a> {
         match *self {
             CompatibleDist::InstalledDist(dist) => ResolvedDistRef::Installed(dist),
-            CompatibleDist::SourceDist(sdist) => ResolvedDistRef::Installable(sdist),
-            CompatibleDist::CompatibleWheel(wheel, _) => ResolvedDistRef::Installable(wheel),
+            CompatibleDist::SourceDist { sdist, prioritized } => {
+                ResolvedDistRef::InstallableRegistrySourceDist { sdist, prioritized }
+            }
+            CompatibleDist::CompatibleWheel {
+                wheel, prioritized, ..
+            } => ResolvedDistRef::InstallableRegistryBuiltDist { wheel, prioritized },
             CompatibleDist::IncompatibleWheel {
-                source_dist: _,
-                wheel,
-            } => ResolvedDistRef::Installable(wheel),
+                wheel, prioritized, ..
+            } => ResolvedDistRef::InstallableRegistryBuiltDist { wheel, prioritized },
         }
     }
 
@@ -302,12 +388,15 @@ impl<'a> CompatibleDist<'a> {
     pub fn for_installation(&self) -> ResolvedDistRef<'a> {
         match *self {
             CompatibleDist::InstalledDist(dist) => ResolvedDistRef::Installed(dist),
-            CompatibleDist::SourceDist(sdist) => ResolvedDistRef::Installable(sdist),
-            CompatibleDist::CompatibleWheel(wheel, _) => ResolvedDistRef::Installable(wheel),
+            CompatibleDist::SourceDist { sdist, prioritized } => {
+                ResolvedDistRef::InstallableRegistrySourceDist { sdist, prioritized }
+            }
+            CompatibleDist::CompatibleWheel {
+                wheel, prioritized, ..
+            } => ResolvedDistRef::InstallableRegistryBuiltDist { wheel, prioritized },
             CompatibleDist::IncompatibleWheel {
-                source_dist,
-                wheel: _,
-            } => ResolvedDistRef::Installable(source_dist),
+                sdist, prioritized, ..
+            } => ResolvedDistRef::InstallableRegistrySourceDist { sdist, prioritized },
         }
     }
 
@@ -316,9 +405,9 @@ impl<'a> CompatibleDist<'a> {
     /// Avoid building source distributions we don't need.
     pub fn prefetchable(&self) -> bool {
         match *self {
-            CompatibleDist::SourceDist(_) => false,
+            CompatibleDist::SourceDist { .. } => false,
             CompatibleDist::InstalledDist(_)
-            | CompatibleDist::CompatibleWheel(_, _)
+            | CompatibleDist::CompatibleWheel { .. }
             | CompatibleDist::IncompatibleWheel { .. } => true,
         }
     }
