@@ -48,7 +48,7 @@ use distribution_types::{
     ParsedUrlError, Requirement, UnresolvedRequirement, UnresolvedRequirementSpecification,
 };
 use pep508_rs::{
-    expand_env_vars, split_scheme, strip_host, Extras, Pep508Error, Pep508ErrorSource,
+    expand_env_vars, split_scheme, strip_host, Extras, MarkerTree, Pep508Error, Pep508ErrorSource,
     RequirementOrigin, Scheme, VerbatimUrl,
 };
 #[cfg(feature = "http")]
@@ -168,6 +168,8 @@ pub struct EditableRequirement {
     pub url: VerbatimUrl,
     /// The extras that should be included when resolving the editable requirements.
     pub extras: Vec<ExtraName>,
+    /// The markers such as `python_version > "3.8"` in `-e ../editable ; python_version > "3.8"`.
+    pub marker: Option<MarkerTree>,
     /// The local path to the editable.
     pub path: PathBuf,
     /// The source file containing the requirement.
@@ -199,6 +201,38 @@ impl EditableRequirement {
         origin: Option<&Path>,
         working_dir: impl AsRef<Path>,
     ) -> Result<Self, RequirementsTxtParserError> {
+        // Identify the markers.
+        let (given, marker) = if let Some((requirement, marker)) = Self::split_markers(given) {
+            let marker = MarkerTree::from_str(marker).map_err(|err| {
+                // Map from error on the markers to error on the whole requirement.
+                let err = Pep508Error {
+                    message: err.message,
+                    start: requirement.len() + err.start,
+                    len: err.len,
+                    input: given.to_string(),
+                };
+                match err.message {
+                    Pep508ErrorSource::String(_) | Pep508ErrorSource::UrlError(_) => {
+                        RequirementsTxtParserError::Pep508 {
+                            start: err.start,
+                            end: err.start + err.len,
+                            source: err,
+                        }
+                    }
+                    Pep508ErrorSource::UnsupportedRequirement(_) => {
+                        RequirementsTxtParserError::UnsupportedRequirement {
+                            start: err.start,
+                            end: err.start + err.len,
+                            source: err,
+                        }
+                    }
+                }
+            })?;
+            (requirement, Some(marker))
+        } else {
+            (given, None)
+        };
+
         // Identify the extras.
         let (requirement, extras) = if let Some((requirement, extras)) = Self::split_extras(given) {
             let extras = Extras::parse(extras).map_err(|err| {
@@ -263,10 +297,15 @@ impl EditableRequirement {
             VerbatimUrl::parse_path(expanded.as_ref(), working_dir.as_ref())
         };
 
+        let url = url.map_err(|err| RequirementsTxtParserError::VerbatimUrl {
+            source: err,
+            url: expanded.to_string(),
+        })?;
+
         // Create a `PathBuf`.
         let path = url
             .to_file_path()
-            .map_err(|()| RequirementsTxtParserError::InvalidEditablePath(expanded.to_string()))?;
+            .map_err(|()| RequirementsTxtParserError::UrlConversion(expanded.to_string()))?;
 
         // Add the verbatim representation of the URL to the `VerbatimUrl`.
         let url = url.with_given(requirement.to_string());
@@ -274,6 +313,7 @@ impl EditableRequirement {
         Ok(Self {
             url,
             extras,
+            marker,
             path,
             origin: origin.map(Path::to_path_buf).map(RequirementOrigin::File),
         })
@@ -300,6 +340,48 @@ impl EditableRequirement {
 
         Some(given.split_at(index))
     }
+
+    /// Identify the markers in an editable URL (e.g., `../editable ; python_version > "3.8"`).
+    pub fn split_markers(given: &str) -> Option<(&str, &str)> {
+        // Take until we see whitespace, unless it's escaped with a backslash, or within brackets
+        // (which would indicate an extra).
+        let mut backslash = false;
+        let mut depth = 0;
+        for (index, c) in given.char_indices() {
+            if backslash {
+                backslash = false;
+            } else if c == '\\' {
+                backslash = true;
+            } else if c == '[' {
+                depth += 1;
+            } else if c == ']' {
+                depth -= 1;
+            } else if depth == 0 && c.is_whitespace() {
+                // We found the end of the requirement; now, find the start of the markers,
+                // delimited by a semicolon.
+                let (requirement, markers) = given.split_at(index);
+
+                // Skip the whitespace.
+                for (index, c) in markers.char_indices() {
+                    if backslash {
+                        backslash = false;
+                    } else if c == '\\' {
+                        backslash = true;
+                    } else if c.is_whitespace() {
+                        continue;
+                    } else if c == ';' {
+                        // The marker starts just after the semicolon.
+                        let markers = &markers[index + 1..];
+                        return Some((requirement, markers));
+                    } else {
+                        // We saw some other character, so this isn't a marker.
+                        return None;
+                    }
+                }
+            }
+        }
+        None
+    }
 }
 
 impl Display for EditableRequirement {
@@ -957,7 +1039,11 @@ pub enum RequirementsTxtParserError {
         start: usize,
         end: usize,
     },
-    InvalidEditablePath(String),
+    VerbatimUrl {
+        source: pep508_rs::VerbatimUrlError,
+        url: String,
+    },
+    UrlConversion(String),
     UnsupportedUrl(String),
     MissingRequirementPrefix(String),
     NoBinary {
@@ -1014,7 +1100,7 @@ impl RequirementsTxtParserError {
     fn with_offset(self, offset: usize) -> Self {
         match self {
             Self::IO(err) => Self::IO(err),
-            Self::InvalidEditablePath(given) => Self::InvalidEditablePath(given),
+            Self::UrlConversion(given) => Self::UrlConversion(given),
             Self::Url {
                 source,
                 url,
@@ -1026,6 +1112,7 @@ impl RequirementsTxtParserError {
                 start: start + offset,
                 end: end + offset,
             },
+            Self::VerbatimUrl { source, url } => Self::VerbatimUrl { source, url },
             Self::UnsupportedUrl(url) => Self::UnsupportedUrl(url),
             Self::MissingRequirementPrefix(given) => Self::MissingRequirementPrefix(given),
             Self::NoBinary {
@@ -1094,12 +1181,15 @@ impl Display for RequirementsTxtParserError {
     fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
         match self {
             Self::IO(err) => err.fmt(f),
-            Self::InvalidEditablePath(given) => {
-                write!(f, "Invalid editable path: {given}")
-            }
             Self::Url { url, start, .. } => {
                 write!(f, "Invalid URL at position {start}: `{url}`")
             }
+            Self::VerbatimUrl { source, url } => {
+                write!(f, "Invalid URL: `{url}`: {source}")
+            }
+            Self::UrlConversion(given) => {
+                write!(f, "Unable to convert URL to path: {given}")
+            }
             Self::UnsupportedUrl(url) => {
                 write!(f, "Unsupported URL (expected a `file://` scheme): `{url}`")
             }
@@ -1154,7 +1244,8 @@ impl std::error::Error for RequirementsTxtParserError {
         match &self {
             Self::IO(err) => err.source(),
             Self::Url { source, .. } => Some(source),
-            Self::InvalidEditablePath(_) => None,
+            Self::VerbatimUrl { source, .. } => Some(source),
+            Self::UrlConversion(_) => None,
             Self::UnsupportedUrl(_) => None,
             Self::MissingRequirementPrefix(_) => None,
             Self::NoBinary { source, .. } => Some(source),
@@ -1183,10 +1274,13 @@ impl Display for RequirementsTxtFileError {
                     self.file.user_display(),
                 )
             }
-            RequirementsTxtParserError::InvalidEditablePath(given) => {
+            RequirementsTxtParserError::VerbatimUrl { url, .. } => {
+                write!(f, "Invalid URL in `{}`: `{url}`", self.file.user_display())
+            }
+            RequirementsTxtParserError::UrlConversion(given) => {
                 write!(
                     f,
-                    "Invalid editable path in `{}`: {given}",
+                    "Unable to convert URL to path `{}`: {given}",
                     self.file.user_display()
                 )
             }
@@ -1427,7 +1521,6 @@ mod test {
     #[test_case(Path::new("poetry-with-hashes.txt"))]
     #[test_case(Path::new("small.txt"))]
     #[test_case(Path::new("whitespace.txt"))]
-    #[test_case(Path::new("editable.txt"))]
     #[tokio::test]
     async fn line_endings(path: &Path) {
         let working_dir = workspace_test_data_dir().join("requirements-txt");
@@ -1469,8 +1562,9 @@ mod test {
 
     #[cfg(unix)]
     #[test_case(Path::new("bare-url.txt"))]
+    #[test_case(Path::new("editable.txt"))]
     #[tokio::test]
-    async fn parse_unnamed_unix(path: &Path) {
+    async fn parse_unix(path: &Path) {
         let working_dir = workspace_test_data_dir().join("requirements-txt");
         let requirements_txt = working_dir.join(path);
 
@@ -1490,8 +1584,9 @@ mod test {
 
     #[cfg(windows)]
     #[test_case(Path::new("bare-url.txt"))]
+    #[test_case(Path::new("editable.txt"))]
     #[tokio::test]
-    async fn parse_unnamed_windows(path: &Path) {
+    async fn parse_windows(path: &Path) {
         let working_dir = workspace_test_data_dir().join("requirements-txt");
         let requirements_txt = working_dir.join(path);
 
@@ -1967,6 +2062,7 @@ mod test {
                             ),
                         },
                         extras: [],
+                        marker: None,
                         path: "/foo/bar",
                         origin: Some(
                             File(
