@@ -3,6 +3,7 @@ pub use crate::source_tree::*;
 pub use crate::sources::*;
 pub use crate::specification::*;
 pub use crate::unnamed::*;
+pub use crate::workspace::*;
 
 mod confirm;
 mod lookahead;
@@ -12,3 +13,4 @@ mod sources;
 mod specification;
 mod unnamed;
 pub mod upgrade;
+mod workspace;
