@@ -1,4 +1,4 @@
-//! Reads the following fields from from `pyproject.toml`:
+//! Reads the following fields from `pyproject.toml`:
 //!
 //! * `project.{dependencies,optional-dependencies}`
 //! * `tool.uv.sources`
@@ -6,7 +6,7 @@
 //!
 //! Then lowers them into a dependency specification.
 
-use std::collections::HashMap;
+use std::collections::BTreeMap;
 use std::io;
 use std::ops::Deref;
 use std::path::{Path, PathBuf};
@@ -59,6 +59,8 @@ pub enum LoweringError {
     InvalidEntry,
     #[error(transparent)]
     InvalidUrl(#[from] url::ParseError),
+    #[error(transparent)]
+    InvalidVerbatimUrl(#[from] pep508_rs::VerbatimUrlError),
     #[error("Can't combine URLs from both `project.dependencies` and `tool.uv.sources`")]
     ConflictingUrls,
     #[error("Could not normalize path: `{0}`")]
@@ -110,7 +112,7 @@ pub struct Tool {
 #[derive(Serialize, Deserialize, Debug, Clone, PartialEq, Eq)]
 #[cfg_attr(feature = "schemars", derive(schemars::JsonSchema))]
 pub struct ToolUv {
-    pub sources: Option<HashMap<PackageName, Source>>,
+    pub sources: Option<BTreeMap<PackageName, Source>>,
     pub workspace: Option<ToolUvWorkspace>,
 }
 
@@ -238,8 +240,8 @@ impl Pep621Metadata {
         extras: &ExtrasSpecification,
         pyproject_path: &Path,
         project_dir: &Path,
-        workspace_sources: &HashMap<PackageName, Source>,
-        workspace_packages: &HashMap<PackageName, String>,
+        workspace_sources: &BTreeMap<PackageName, Source>,
+        workspace_packages: &BTreeMap<PackageName, String>,
         preview: PreviewMode,
     ) -> Result<Option<Self>, Pep621Error> {
         let project_sources = pyproject
@@ -323,9 +325,9 @@ pub(crate) fn lower_requirements(
     pyproject_path: &Path,
     project_name: &PackageName,
     project_dir: &Path,
-    project_sources: &HashMap<PackageName, Source>,
-    workspace_sources: &HashMap<PackageName, Source>,
-    workspace_packages: &HashMap<PackageName, String>,
+    project_sources: &BTreeMap<PackageName, Source>,
+    workspace_sources: &BTreeMap<PackageName, Source>,
+    workspace_packages: &BTreeMap<PackageName, String>,
     preview: PreviewMode,
 ) -> Result<Requirements, Pep621Error> {
     let dependencies = dependencies
@@ -386,9 +388,9 @@ pub(crate) fn lower_requirement(
     requirement: pep508_rs::Requirement,
     project_name: &PackageName,
     project_dir: &Path,
-    project_sources: &HashMap<PackageName, Source>,
-    workspace_sources: &HashMap<PackageName, Source>,
-    workspace_packages: &HashMap<PackageName, String>,
+    project_sources: &BTreeMap<PackageName, Source>,
+    workspace_sources: &BTreeMap<PackageName, Source>,
+    workspace_packages: &BTreeMap<PackageName, String>,
     preview: PreviewMode,
 ) -> Result<Requirement, LoweringError> {
     let source = project_sources
@@ -496,7 +498,7 @@ pub(crate) fn lower_requirement(
             if matches!(requirement.version_or_url, Some(VersionOrUrl::Url(_))) {
                 return Err(LoweringError::ConflictingUrls);
             }
-            path_source(path, project_dir, editable)?
+            path_source(path, project_dir, editable.unwrap_or(false))?
         }
         Source::Registry { index } => match requirement.version_or_url {
             None => {
@@ -529,7 +531,7 @@ pub(crate) fn lower_requirement(
                 .get(&requirement.name)
                 .ok_or(LoweringError::UndeclaredWorkspacePackage)?
                 .clone();
-            path_source(path, project_dir, editable)?
+            path_source(path, project_dir, editable.unwrap_or(true))?
         }
         Source::CatchAll { .. } => {
             // Emit a dedicated error message, which is an improvement over Serde's default error.
@@ -549,9 +551,9 @@ pub(crate) fn lower_requirement(
 fn path_source(
     path: String,
     project_dir: &Path,
-    editable: Option<bool>,
+    editable: bool,
 ) -> Result<RequirementSource, LoweringError> {
-    let url = VerbatimUrl::parse_path(&path, project_dir).with_given(path.clone());
+    let url = VerbatimUrl::parse_path(&path, project_dir)?.with_given(path.clone());
     let path_buf = PathBuf::from(&path);
     let path_buf = path_buf
         .absolutize_from(project_dir)
