@@ -43,8 +43,8 @@ use crate::manifest::Manifest;
 use crate::pins::FilePins;
 use crate::preferences::Preferences;
 use crate::pubgrub::{
-    PubGrubDependencies, PubGrubDistribution, PubGrubPackage, PubGrubPriorities, PubGrubPython,
-    PubGrubRequirement, PubGrubSpecifier,
+    PubGrubDependencies, PubGrubDistribution, PubGrubPackage, PubGrubPackageInner,
+    PubGrubPriorities, PubGrubPython, PubGrubRequirement, PubGrubSpecifier,
 };
 use crate::python_requirement::PythonRequirement;
 use crate::resolution::ResolutionGraph;
@@ -401,7 +401,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
         visited: &mut FxHashSet<PackageName>,
         request_sink: Sender<Request>,
     ) -> Result<ResolutionGraph, ResolveError> {
-        let root = PubGrubPackage::Root(self.project.clone());
+        let root = PubGrubPackage::from(PubGrubPackageInner::Root(self.project.clone()));
         let mut prefetcher = BatchPrefetcher::default();
         let mut state = SolveState {
             pubgrub: State::init(root.clone(), MIN_VERSION.clone()),
@@ -480,8 +480,8 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                         .expect("a package was chosen but we don't have a term.");
 
                     // Check if the decision was due to the package being unavailable
-                    if let PubGrubPackage::Package(ref package_name, _, _) = state.next {
-                        if let Some(entry) = self.unavailable_packages.get(package_name) {
+                    if let PubGrubPackageInner::Package { ref name, .. } = &*state.next {
+                        if let Some(entry) = self.unavailable_packages.get(name) {
                             state
                                 .pubgrub
                                 .add_incompatibility(Incompatibility::custom_term(
@@ -530,7 +530,10 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                                 .add_incompatibility(Incompatibility::from_dependency(
                                     package.clone(),
                                     Range::singleton(version.clone()),
-                                    (PubGrubPackage::Python(kind), python_version.clone()),
+                                    (
+                                        PubGrubPackage::from(PubGrubPackageInner::Python(kind)),
+                                        python_version.clone(),
+                                    ),
                                 ));
                         }
                         state
@@ -633,11 +636,13 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
         package: &PubGrubPackage,
         request_sink: &Sender<Request>,
     ) -> Result<(), ResolveError> {
-        match package {
-            PubGrubPackage::Root(_) => {}
-            PubGrubPackage::Python(_) => {}
-            PubGrubPackage::Extra(_, _, _) => {}
-            PubGrubPackage::Package(name, _extra, None) => {
+        match &**package {
+            PubGrubPackageInner::Root(_) => {}
+            PubGrubPackageInner::Python(_) => {}
+            PubGrubPackageInner::Extra { .. } => {}
+            PubGrubPackageInner::Package {
+                name, url: None, ..
+            } => {
                 // Verify that the package is allowed under the hash-checking policy.
                 if !self.hasher.allows_package(name) {
                     return Err(ResolveError::UnhashedPackage(name.clone()));
@@ -648,7 +653,11 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     request_sink.blocking_send(Request::Package(name.clone()))?;
                 }
             }
-            PubGrubPackage::Package(name, _extra, Some(url)) => {
+            PubGrubPackageInner::Package {
+                name,
+                url: Some(url),
+                ..
+            } => {
                 // Verify that the package is allowed under the hash-checking policy.
                 if !self.hasher.allows_url(&url.verbatim) {
                     return Err(ResolveError::UnhashedPackage(name.clone()));
@@ -678,10 +687,16 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
         // Iterate over the potential packages, and fetch file metadata for any of them. These
         // represent our current best guesses for the versions that we _might_ select.
         for (package, range) in packages {
-            let PubGrubPackage::Package(package_name, None, None) = package else {
+            let PubGrubPackageInner::Package {
+                name,
+                extra: None,
+                marker: None,
+                url: None,
+            } = &**package
+            else {
                 continue;
             };
-            request_sink.blocking_send(Request::Prefetch(package_name.clone(), range.clone()))?;
+            request_sink.blocking_send(Request::Prefetch(name.clone(), range.clone()))?;
         }
         Ok(())
     }
@@ -699,10 +714,12 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
         visited: &mut FxHashSet<PackageName>,
         request_sink: &Sender<Request>,
     ) -> Result<Option<ResolverVersion>, ResolveError> {
-        match package {
-            PubGrubPackage::Root(_) => Ok(Some(ResolverVersion::Available(MIN_VERSION.clone()))),
+        match &**package {
+            PubGrubPackageInner::Root(_) => {
+                Ok(Some(ResolverVersion::Available(MIN_VERSION.clone())))
+            }
 
-            PubGrubPackage::Python(PubGrubPython::Installed) => {
+            PubGrubPackageInner::Python(PubGrubPython::Installed) => {
                 let version = self.python_requirement.installed();
                 if range.contains(version) {
                     Ok(Some(ResolverVersion::Available(version.deref().clone())))
@@ -711,7 +728,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                 }
             }
 
-            PubGrubPackage::Python(PubGrubPython::Target) => {
+            PubGrubPackageInner::Python(PubGrubPython::Target) => {
                 let version = self.python_requirement.target();
                 if range.contains(version) {
                     Ok(Some(ResolverVersion::Available(version.deref().clone())))
@@ -720,15 +737,23 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                 }
             }
 
-            PubGrubPackage::Extra(package_name, _, Some(url))
-            | PubGrubPackage::Package(package_name, _, Some(url)) => {
+            PubGrubPackageInner::Extra {
+                name,
+                url: Some(url),
+                ..
+            }
+            | PubGrubPackageInner::Package {
+                name,
+                url: Some(url),
+                ..
+            } => {
                 debug!(
                     "Searching for a compatible version of {package} @ {} ({range})",
                     url.verbatim
                 );
 
                 // If the dist is an editable, return the version from the editable metadata.
-                if let Some(editable) = self.editables.get(package_name) {
+                if let Some(editable) = self.editables.get(name) {
                     let version = &editable.metadata.version;
 
                     // The version is incompatible with the requirement.
@@ -752,7 +777,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     return Ok(Some(ResolverVersion::Available(version.clone())));
                 }
 
-                let dist = PubGrubDistribution::from_url(package_name, url);
+                let dist = PubGrubDistribution::from_url(name, url);
                 let response = self
                     .index
                     .distributions()
@@ -764,26 +789,26 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     MetadataResponse::Found(archive) => &archive.metadata,
                     MetadataResponse::Offline => {
                         self.unavailable_packages
-                            .insert(package_name.clone(), UnavailablePackage::Offline);
+                            .insert(name.clone(), UnavailablePackage::Offline);
                         return Ok(None);
                     }
                     MetadataResponse::InvalidMetadata(err) => {
                         self.unavailable_packages.insert(
-                            package_name.clone(),
+                            name.clone(),
                             UnavailablePackage::InvalidMetadata(err.to_string()),
                         );
                         return Ok(None);
                     }
                     MetadataResponse::InconsistentMetadata(err) => {
                         self.unavailable_packages.insert(
-                            package_name.clone(),
+                            name.clone(),
                             UnavailablePackage::InvalidMetadata(err.to_string()),
                         );
                         return Ok(None);
                     }
                     MetadataResponse::InvalidStructure(err) => {
                         self.unavailable_packages.insert(
-                            package_name.clone(),
+                            name.clone(),
                             UnavailablePackage::InvalidStructure(err.to_string()),
                         );
                         return Ok(None);
@@ -813,31 +838,35 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                 Ok(Some(ResolverVersion::Available(version.clone())))
             }
 
-            PubGrubPackage::Extra(package_name, _, None)
-            | PubGrubPackage::Package(package_name, _, None) => {
+            PubGrubPackageInner::Extra {
+                name, url: None, ..
+            }
+            | PubGrubPackageInner::Package {
+                name, url: None, ..
+            } => {
                 // Wait for the metadata to be available.
                 let versions_response = self
                     .index
                     .packages()
-                    .wait_blocking(package_name)
+                    .wait_blocking(name)
                     .ok_or(ResolveError::Unregistered)?;
-                visited.insert(package_name.clone());
+                visited.insert(name.clone());
 
                 let version_maps = match *versions_response {
                     VersionsResponse::Found(ref version_maps) => version_maps.as_slice(),
                     VersionsResponse::NoIndex => {
                         self.unavailable_packages
-                            .insert(package_name.clone(), UnavailablePackage::NoIndex);
+                            .insert(name.clone(), UnavailablePackage::NoIndex);
                         &[]
                     }
                     VersionsResponse::Offline => {
                         self.unavailable_packages
-                            .insert(package_name.clone(), UnavailablePackage::Offline);
+                            .insert(name.clone(), UnavailablePackage::Offline);
                         &[]
                     }
                     VersionsResponse::NotFound => {
                         self.unavailable_packages
-                            .insert(package_name.clone(), UnavailablePackage::NotFound);
+                            .insert(name.clone(), UnavailablePackage::NotFound);
                         &[]
                     }
                 };
@@ -846,7 +875,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
 
                 // Find a version.
                 let Some(candidate) = self.selector.select(
-                    package_name,
+                    name,
                     range,
                     version_maps,
                     &self.preferences,
@@ -892,7 +921,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                 let version = candidate.version().clone();
 
                 // Emit a request to fetch the metadata for this version.
-                if matches!(package, PubGrubPackage::Package(_, _, _)) {
+                if matches!(&**package, PubGrubPackageInner::Package { .. }) {
                     if self.index.distributions().register(candidate.version_id()) {
                         let request = Request::from(dist.for_resolution());
                         request_sink.blocking_send(request)?;
@@ -913,8 +942,8 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
         priorities: &mut PubGrubPriorities,
         request_sink: &Sender<Request>,
     ) -> Result<Dependencies, ResolveError> {
-        match package {
-            PubGrubPackage::Root(_) => {
+        match &**package {
+            PubGrubPackageInner::Root(_) => {
                 // Add the root requirements.
                 let dependencies = PubGrubDependencies::from_requirements(
                     &self.requirements,
@@ -951,6 +980,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     let package = PubGrubPackage::from_package(
                         editable.metadata.name.clone(),
                         None,
+                        None,
                         &self.urls,
                     );
                     let version = Range::singleton(editable.metadata.version.clone());
@@ -967,6 +997,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                             PubGrubPackage::from_package(
                                 editable.metadata.name.clone(),
                                 Some(extra.clone()),
+                                None,
                                 &self.urls,
                             ),
                             Range::singleton(editable.metadata.version.clone()),
@@ -995,18 +1026,23 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                 Ok(Dependencies::Available(dependencies.into()))
             }
 
-            PubGrubPackage::Python(_) => Ok(Dependencies::Available(Vec::default())),
+            PubGrubPackageInner::Python(_) => Ok(Dependencies::Available(Vec::default())),
 
-            PubGrubPackage::Package(package_name, extra, url) => {
+            PubGrubPackageInner::Package {
+                name,
+                extra,
+                marker: _marker,
+                url,
+            } => {
                 // If we're excluding transitive dependencies, short-circuit.
                 if self.dependency_mode.is_direct() {
                     // If an extra is provided, wait for the metadata to be available, since it's
                     // still required for generating the lock file.
-                    if !self.editables.contains(package_name) {
+                    if !self.editables.contains(name) {
                         // Determine the distribution to lookup.
                         let dist = match url {
-                            Some(url) => PubGrubDistribution::from_url(package_name, url),
-                            None => PubGrubDistribution::from_registry(package_name, version),
+                            Some(url) => PubGrubDistribution::from_url(name, url),
+                            None => PubGrubDistribution::from_registry(name, version),
                         };
                         let version_id = dist.version_id();
 
@@ -1021,7 +1057,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                 }
 
                 // Determine if the distribution is editable.
-                if let Some(editable) = self.editables.get(package_name) {
+                if let Some(editable) = self.editables.get(name) {
                     let requirements: Vec<_> = editable
                         .metadata
                         .requires_dist
@@ -1033,7 +1069,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                         &requirements,
                         &self.constraints,
                         &self.overrides,
-                        Some(package_name),
+                        Some(name),
                         extra.as_ref(),
                         &self.urls,
                         &self.locals,
@@ -1055,24 +1091,21 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
 
                 // Determine the distribution to lookup.
                 let dist = match url {
-                    Some(url) => PubGrubDistribution::from_url(package_name, url),
-                    None => PubGrubDistribution::from_registry(package_name, version),
+                    Some(url) => PubGrubDistribution::from_url(name, url),
+                    None => PubGrubDistribution::from_registry(name, version),
                 };
                 let version_id = dist.version_id();
 
                 // If the package does not exist in the registry or locally, we cannot fetch its dependencies
-                if self.unavailable_packages.get(package_name).is_some()
-                    && self
-                        .installed_packages
-                        .get_packages(package_name)
-                        .is_empty()
+                if self.unavailable_packages.get(name).is_some()
+                    && self.installed_packages.get_packages(name).is_empty()
                 {
                     debug_assert!(
                         false,
                         "Dependencies were requested for a package that is not available"
                     );
                     return Err(ResolveError::Failure(format!(
-                        "The package is unavailable: {package_name}"
+                        "The package is unavailable: {name}"
                     )));
                 }
 
@@ -1087,15 +1120,15 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     MetadataResponse::Found(archive) => &archive.metadata,
                     MetadataResponse::Offline => {
                         self.incomplete_packages
-                            .entry(package_name.clone())
+                            .entry(name.clone())
                             .or_default()
                             .insert(version.clone(), IncompletePackage::Offline);
                         return Ok(Dependencies::Unavailable(UnavailableVersion::Offline));
                     }
                     MetadataResponse::InvalidMetadata(err) => {
-                        warn!("Unable to extract metadata for {package_name}: {err}");
+                        warn!("Unable to extract metadata for {name}: {err}");
                         self.incomplete_packages
-                            .entry(package_name.clone())
+                            .entry(name.clone())
                             .or_default()
                             .insert(
                                 version.clone(),
@@ -1106,9 +1139,9 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                         ));
                     }
                     MetadataResponse::InconsistentMetadata(err) => {
-                        warn!("Unable to extract metadata for {package_name}: {err}");
+                        warn!("Unable to extract metadata for {name}: {err}");
                         self.incomplete_packages
-                            .entry(package_name.clone())
+                            .entry(name.clone())
                             .or_default()
                             .insert(
                                 version.clone(),
@@ -1119,9 +1152,9 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                         ));
                     }
                     MetadataResponse::InvalidStructure(err) => {
-                        warn!("Unable to extract metadata for {package_name}: {err}");
+                        warn!("Unable to extract metadata for {name}: {err}");
                         self.incomplete_packages
-                            .entry(package_name.clone())
+                            .entry(name.clone())
                             .or_default()
                             .insert(
                                 version.clone(),
@@ -1143,7 +1176,7 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
                     &requirements,
                     &self.constraints,
                     &self.overrides,
-                    Some(package_name),
+                    Some(name),
                     extra.as_ref(),
                     &self.urls,
                     &self.locals,
@@ -1164,13 +1197,28 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
             }
 
             // Add a dependency on both the extra and base package.
-            PubGrubPackage::Extra(package_name, extra, url) => Ok(Dependencies::Available(vec![
+            PubGrubPackageInner::Extra {
+                name,
+                extra,
+                marker: _marker,
+                url,
+            } => Ok(Dependencies::Available(vec![
                 (
-                    PubGrubPackage::Package(package_name.clone(), None, url.clone()),
+                    PubGrubPackage::from(PubGrubPackageInner::Package {
+                        name: name.clone(),
+                        extra: None,
+                        marker: None,
+                        url: url.clone(),
+                    }),
                     Range::singleton(version.clone()),
                 ),
                 (
-                    PubGrubPackage::Package(package_name.clone(), Some(extra.clone()), url.clone()),
+                    PubGrubPackage::from(PubGrubPackageInner::Package {
+                        name: name.clone(),
+                        extra: Some(extra.clone()),
+                        marker: None,
+                        url: url.clone(),
+                    }),
                     Range::singleton(version.clone()),
                 ),
             ])),
@@ -1395,15 +1443,21 @@ impl<InstalledPackages: InstalledPackagesProvider> ResolverState<InstalledPackag
 
     fn on_progress(&self, package: &PubGrubPackage, version: &Version) {
         if let Some(reporter) = self.reporter.as_ref() {
-            match package {
-                PubGrubPackage::Root(_) => {}
-                PubGrubPackage::Python(_) => {}
-                PubGrubPackage::Extra(_, _, _) => {}
-                PubGrubPackage::Package(package_name, _extra, Some(url)) => {
-                    reporter.on_progress(package_name, &VersionOrUrlRef::Url(&url.verbatim));
+            match &**package {
+                PubGrubPackageInner::Root(_) => {}
+                PubGrubPackageInner::Python(_) => {}
+                PubGrubPackageInner::Extra { .. } => {}
+                PubGrubPackageInner::Package {
+                    name,
+                    url: Some(url),
+                    ..
+                } => {
+                    reporter.on_progress(name, &VersionOrUrlRef::Url(&url.verbatim));
                 }
-                PubGrubPackage::Package(package_name, _extra, None) => {
-                    reporter.on_progress(package_name, &VersionOrUrlRef::Version(version));
+                PubGrubPackageInner::Package {
+                    name, url: None, ..
+                } => {
+                    reporter.on_progress(name, &VersionOrUrlRef::Version(version));
                 }
             }
         }
