@@ -10,7 +10,7 @@ use anyhow::Result;
 use chrono::{DateTime, Utc};
 use once_cell::sync::Lazy;
 
-use distribution_types::{IndexLocations, Requirement, Resolution, SourceDist};
+use distribution_types::{CachedDist, IndexLocations, Requirement, Resolution, SourceDist};
 use pep508_rs::{MarkerEnvironment, MarkerEnvironmentBuilder};
 use platform_tags::{Arch, Os, Platform, Tags};
 use uv_cache::Cache;
@@ -19,7 +19,7 @@ use uv_configuration::{
     BuildKind, Concurrency, Constraints, NoBinary, NoBuild, Overrides, SetupPyStrategy,
 };
 use uv_distribution::DistributionDatabase;
-use uv_interpreter::{find_default_python, Interpreter, PythonEnvironment};
+use uv_interpreter::{find_default_interpreter, Interpreter, PythonEnvironment};
 use uv_resolver::{
     DisplayResolutionGraph, ExcludeNewer, Exclusions, FlatIndex, InMemoryIndex, Manifest, Options,
     OptionsBuilder, PreReleaseMode, Preference, PythonRequirement, ResolutionGraph, ResolutionMode,
@@ -89,7 +89,11 @@ impl BuildContext for DummyContext {
         panic!("The test should not need to build source distributions")
     }
 
-    async fn install<'a>(&'a self, _: &'a Resolution, _: &'a PythonEnvironment) -> Result<()> {
+    async fn install<'a>(
+        &'a self,
+        _: &'a Resolution,
+        _: &'a PythonEnvironment,
+    ) -> Result<Vec<CachedDist>> {
         panic!("The test should not need to build source distributions")
     }
 
@@ -124,10 +128,13 @@ async fn resolve(
     tags: &Tags,
 ) -> Result<ResolutionGraph> {
     let cache = Cache::temp().unwrap().init().unwrap();
-    let real_interpreter = find_default_python(&cache).expect("Expected a python to be installed");
     let client = RegistryClientBuilder::new(cache).build();
     let flat_index = FlatIndex::default();
     let index = InMemoryIndex::default();
+    let real_interpreter = find_default_interpreter(&Cache::temp().unwrap())
+        .unwrap()
+        .expect("Python should be installed")
+        .into_interpreter();
     let interpreter = Interpreter::artificial(real_interpreter.platform().clone(), markers.clone());
     let python_requirement = PythonRequirement::from_marker_environment(&interpreter, markers);
     let cache = Cache::temp().unwrap().init().unwrap();
