@@ -5,16 +5,17 @@ use install_wheel_rs::linker::LinkMode;
 use uv_cache::Cache;
 use uv_client::RegistryClientBuilder;
 use uv_configuration::{
-    Concurrency, ConfigSettings, NoBinary, NoBuild, PreviewMode, SetupPyStrategy,
+    Concurrency, ConfigSettings, NoBinary, NoBuild, PreviewMode, Reinstall, SetupPyStrategy,
 };
 use uv_dispatch::BuildDispatch;
 use uv_installer::SitePackages;
+use uv_requirements::ProjectWorkspace;
 use uv_resolver::{FlatIndex, InMemoryIndex, Lock};
 use uv_types::{BuildIsolation, HashStrategy, InFlight};
 use uv_warnings::warn_user;
 
-use crate::commands::project::discovery::Project;
-use crate::commands::{project, ExitStatus};
+use crate::commands::pip::operations::Modifications;
+use crate::commands::{pip, project, ExitStatus};
 use crate::editables::ResolvedEditables;
 use crate::printer::Printer;
 
@@ -30,22 +31,19 @@ pub(crate) async fn sync(
     }
 
     // Find the project requirements.
-    let Some(project) = Project::find(std::env::current_dir()?)? else {
-        return Err(anyhow::anyhow!(
-            "Unable to find `pyproject.toml` for project."
-        ));
-    };
+    let project = ProjectWorkspace::discover(std::env::current_dir()?)?;
 
     // Discover or create the virtual environment.
-    let venv = project::init(&project, cache, printer)?;
+    let venv = project::init_environment(&project, cache, printer)?;
     let markers = venv.interpreter().markers();
     let tags = venv.interpreter().tags()?;
 
     // Read the lockfile.
     let resolution = {
-        let encoded = fs_err::tokio::read_to_string(project.root().join("uv.lock")).await?;
+        let encoded =
+            fs_err::tokio::read_to_string(project.workspace().root().join("uv.lock")).await?;
         let lock: Lock = toml::from_str(&encoded)?;
-        lock.to_resolution(markers, tags, project.name())
+        lock.to_resolution(markers, tags, project.project_name())
     };
 
     // Initialize the registry client.
@@ -57,7 +55,10 @@ pub(crate) async fn sync(
 
     // TODO(charlie): Respect project configuration.
     let build_isolation = BuildIsolation::default();
+    let compile = false;
+    let concurrency = Concurrency::default();
     let config_settings = ConfigSettings::default();
+    let dry_run = false;
     let flat_index = FlatIndex::default();
     let hasher = HashStrategy::default();
     let in_flight = InFlight::default();
@@ -66,8 +67,8 @@ pub(crate) async fn sync(
     let link_mode = LinkMode::default();
     let no_binary = NoBinary::default();
     let no_build = NoBuild::default();
+    let reinstall = Reinstall::default();
     let setup_py = SetupPyStrategy::default();
-    let concurrency = Concurrency::default();
 
     // Create a build dispatch.
     let build_dispatch = BuildDispatch::new(
@@ -87,30 +88,52 @@ pub(crate) async fn sync(
         concurrency,
     );
 
-    // TODO(konsti): Read editables from lockfile.
-    let editables = ResolvedEditables::default();
+    let site_packages = SitePackages::from_executable(&venv)?;
+
+    // Build any editables.
+    let editables = ResolvedEditables::resolve(
+        resolution.editables(),
+        &site_packages,
+        &reinstall,
+        &hasher,
+        venv.interpreter(),
+        tags,
+        cache,
+        &client,
+        &build_dispatch,
+        concurrency,
+        printer,
+    )
+    .await?;
 
     let site_packages = SitePackages::from_executable(&venv)?;
 
     // Sync the environment.
-    project::install(
+    pip::operations::install(
         &resolution,
-        editables,
+        &editables,
         site_packages,
+        Modifications::Sufficient,
+        &reinstall,
         &no_binary,
         link_mode,
+        compile,
         &index_locations,
         &hasher,
         tags,
         &client,
         &in_flight,
+        concurrency,
         &build_dispatch,
         cache,
         &venv,
+        dry_run,
         printer,
-        concurrency,
     )
     .await?;
 
+    // Notify the user of any resolution diagnostics.
+    pip::operations::diagnose_resolution(resolution.diagnostics(), printer)?;
+
     Ok(ExitStatus::Success)
 }
