@@ -5,12 +5,11 @@ use std::vec;
 
 use anstream::eprint;
 use anyhow::Result;
-use itertools::Itertools;
 use miette::{Diagnostic, IntoDiagnostic};
 use owo_colors::OwoColorize;
 use thiserror::Error;
 
-use distribution_types::{DistributionMetadata, IndexLocations, Name, Requirement, ResolvedDist};
+use distribution_types::{IndexLocations, Requirement};
 use install_wheel_rs::linker::LinkMode;
 use uv_auth::store_credentials_from_url;
 use uv_cache::Cache;
@@ -19,11 +18,13 @@ use uv_configuration::{Concurrency, KeyringProviderType};
 use uv_configuration::{ConfigSettings, IndexStrategy, NoBinary, NoBuild, SetupPyStrategy};
 use uv_dispatch::BuildDispatch;
 use uv_fs::Simplified;
-use uv_interpreter::{find_default_python, find_requested_python, Error};
+use uv_interpreter::{
+    find_default_interpreter, find_interpreter, InterpreterRequest, SourceSelector,
+};
 use uv_resolver::{ExcludeNewer, FlatIndex, InMemoryIndex, OptionsBuilder};
 use uv_types::{BuildContext, BuildIsolation, HashStrategy, InFlight};
 
-use crate::commands::ExitStatus;
+use crate::commands::{pip, ExitStatus};
 use crate::printer::Printer;
 use crate::shell::Shell;
 
@@ -116,14 +117,17 @@ async fn venv_impl(
     printer: Printer,
 ) -> miette::Result<ExitStatus> {
     // Locate the Python interpreter.
-    let interpreter = if let Some(python_request) = python_request {
-        find_requested_python(python_request, cache)
-            .into_diagnostic()?
-            .ok_or(Error::NoSuchPython(python_request.to_string()))
-            .into_diagnostic()?
+    let interpreter = if let Some(python) = python_request.as_ref() {
+        let system = uv_interpreter::SystemPython::Required;
+        let request = InterpreterRequest::parse(python);
+        let sources = SourceSelector::from_settings(system);
+        find_interpreter(&request, system, &sources, cache)
     } else {
-        find_default_python(cache).into_diagnostic()?
-    };
+        find_default_interpreter(cache)
+    }
+    .into_diagnostic()?
+    .into_diagnostic()?
+    .into_interpreter();
 
     // Add all authenticated sources to the cache.
     for url in index_locations.urls() {
@@ -218,50 +222,36 @@ async fn venv_impl(
         .with_options(OptionsBuilder::new().exclude_newer(exclude_newer).build());
 
         // Resolve the seed packages.
-        let mut requirements =
+        let requirements = if interpreter.python_tuple() < (3, 12) {
+            // Only include `setuptools` and `wheel` on Python <3.12
             vec![
                 Requirement::from_pep508(pep508_rs::Requirement::from_str("pip").unwrap()).unwrap(),
-            ];
-
-        // Only include `setuptools` and `wheel` on Python <3.12
-        if interpreter.python_tuple() < (3, 12) {
-            requirements.push(
                 Requirement::from_pep508(pep508_rs::Requirement::from_str("setuptools").unwrap())
                     .unwrap(),
-            );
-            requirements.push(
                 Requirement::from_pep508(pep508_rs::Requirement::from_str("wheel").unwrap())
                     .unwrap(),
-            );
-        }
+            ]
+        } else {
+            vec![
+                Requirement::from_pep508(pep508_rs::Requirement::from_str("pip").unwrap()).unwrap(),
+            ]
+        };
+
+        // Resolve and install the requirements.
+        //
+        // Since the virtual environment is empty, and the set of requirements is trivial (no
+        // constraints, no editables, etc.), we can use the build dispatch APIs directly.
         let resolution = build_dispatch
             .resolve(&requirements)
             .await
             .map_err(VenvError::Seed)?;
-
-        // Install into the environment.
-        build_dispatch
+        let installed = build_dispatch
             .install(&resolution, &venv)
             .await
             .map_err(VenvError::Seed)?;
 
-        for distribution in resolution
-            .distributions()
-            .filter_map(|dist| match dist {
-                ResolvedDist::Installable(dist) => Some(dist),
-                ResolvedDist::Installed(_) => None,
-            })
-            .sorted_unstable_by(|a, b| a.name().cmp(b.name()).then(a.version().cmp(&b.version())))
-        {
-            writeln!(
-                printer.stderr(),
-                " {} {}{}",
-                "+".green(),
-                distribution.name().as_ref().bold(),
-                distribution.version_or_url().dimmed()
-            )
+        pip::operations::report_modifications(installed, Vec::new(), Vec::new(), printer)
             .into_diagnostic()?;
-        }
     }
 
     // Determine the appropriate activation command.
